/* -*- c++ -*- */
/* 
 * Copyright 2018 Viridiana Mares, Facultad de Ingeniería UNAM.
 * 
 * This is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street,
 * Boston, MA 02110-1301, USA.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <gnuradio/io_signature.h>
#include "ldpc_encoder_bb_impl.h"

namespace gr {
  namespace atsc32 {
      using namespace std;

    ldpc_encoder_bb::sptr
    ldpc_encoder_bb::make(atsc3_framesize_t framesize, atsc3_code_rate_t rate)
    {
      return gnuradio::get_initial_sptr
        (new ldpc_encoder_bb_impl(framesize, rate));
    }

    /*
     * The private constructor
     */
    ldpc_encoder_bb_impl::ldpc_encoder_bb_impl(atsc3_framesize_t framesize, atsc3_code_rate_t rate)
      : gr::block("ldpc_encoder_bb",
              gr::io_signature::make(1, 1, sizeof(unsigned char)),
              gr::io_signature::make(1, 1, sizeof(unsigned char)))
    {
        if (framesize == gr::atsc32::FECFRAME_NORMAL)
        {
            frame_size = FRAME_SIZE_NORMAL;
            switch (rate)
            {
                case gr::atsc32::C6_15:
                    nouter = 25920;
                    q_val = 108;
                    break;

                //Se asignan valores por default, no existe tasa de 7/15 para un frmae de tamaño normal
                case gr::atsc32::C7_15:
                    nouter = 7560;
                    q_val = 24;
                    break;   

                case gr::atsc32::C8_15:
                    nouter = 34560;
                    q_val = 84;
                    break;

                case gr::atsc32::C9_15:
                    nouter = 38880;
                    q_val = 72;
                    break;

                case gr::atsc32::C10_15:
                    nouter = 43200;
                    q_val = 60;
                    break;

                case gr::atsc32::C11_15:
                    nouter = 47520;
                    q_val = 48;
                    break;
            }

        }
        else
        {
            frame_size = FRAME_SIZE_SHORT;
            switch (rate)
            {
                case gr::atsc32::C6_15:
                    nouter = 6480;
                    q_val = 27;
                    break;

                case gr::atsc32::C7_15:
                    nouter = 7560;
                    q_val = 24;
                    break;

                case gr::atsc32::C8_15:
                    nouter = 8640;
                    q_val = 21;
                    break;

                case gr::atsc32::C9_15:
                    nouter = 9720;
                    q_val = 18;
                    break;

                case gr::atsc32::C10_15:
                    nouter = 10800;
                    q_val = 15;
                    break;

                case gr::atsc32::C11_15:
                    nouter = 11880;
                    q_val = 12;
                    break;
            }
        }
        code_rate = rate;
        ldpc_lookup_generate();
        set_output_multiple(frame_size);
    }

    /*
     * Our virtual destructor.
     */
    ldpc_encoder_bb_impl::~ldpc_encoder_bb_impl()
    {
    }

    void
    ldpc_encoder_bb_impl::forecast (int noutput_items, gr_vector_int &ninput_items_required)
    {
        ninput_items_required[0] = (noutput_items / frame_size) * nouter;
    }

/*
 * Codificación tipo B o IRA:
 * q(i,j,l)=q(i,j,0)+Qldpc*l(mod Minner), para l de 0 a 360
 * l=k(mod 360); se hace 0 en 360 y multiplos de 360, de tal forma que cuando se cambia de fila se toman las direcciones tal y como vienen en la tabla
 * i=|_k/360_|; es cero hasta que k=360, cambia a una nueva fila cada 360 datos
 */

#define LDPC_BF(TABLE_COD, ROWS) \
for (int row = 0; row < ROWS; row++) \
{ \
    for(int l = 0; l < 360; l++) \
    { \
        for (int col = 1; col <= TABLE_COD[row][0]; col++) \
        { \
            ldpc_encode.p[index] = (TABLE_COD[row][col] + (l * q)) % pbits; \
            ldpc_encode.d[index] = im; \
            index++; \
        } \
        im++; \
    } \
}

void ldpc_encoder_bb_impl::ldpc_lookup_generate()
{
    int im;
    int index;
    int pbits;
    int q;
    index = 0;
    im = 0;

    pbits = frame_size - nouter;    //number of parity bits
    q = q_val;

    if (frame_size == FRAME_SIZE_NORMAL)
    {
        cout << ">>> Running a LDPC encoder" << endl;
        cout << "Code length: Normal" << endl;

        if (code_rate == gr::atsc32::C6_15) {
            LDPC_BF(ldpc_tab_6_15N, 72);
            cout << "Encoding at 6/15" << endl;
        }

        if (code_rate == gr::atsc32::C7_15) {
            cout << "There is no coding rate of 7/15 for a normal code length, the signal has not been processed correctly" << endl;
        }

        if (code_rate == gr::atsc32::C8_15)  {
            LDPC_BF(ldpc_tab_8_15N, 96);
            cout << "Encoding at 8/15" << endl;
        }

        if (code_rate == gr::atsc32::C9_15)  {
            LDPC_BF(ldpc_tab_9_15N, 108);
            cout << "Encoding at 9/15" << endl;
        }

        if (code_rate == gr::atsc32::C10_15)  {
            LDPC_BF(ldpc_tab_10_15N, 120);
            cout << "Encoding at 10/15" << endl;
        }

        if (code_rate == gr::atsc32::C11_15)  {
            LDPC_BF(ldpc_tab_11_15N, 132);
            cout << "Encoding at 11/15" << endl;
        }
    }
    else
    {
        cout << ">>> Running a LDPC encoder" << endl;
        cout << "Code length: Short" << endl;
        if (code_rate == gr::atsc32::C6_15) {
            LDPC_BF(ldpc_tab_6_15S, 18);
            cout << "Encoding at 6/15" << endl;
        }

        if (code_rate == gr::atsc32::C7_15) {
            LDPC_BF(ldpc_tab_7_15S, 21);
            cout << "Encoding at 7/15" << endl;
        }
        if (code_rate == gr::atsc32::C8_15)  {
            LDPC_BF(ldpc_tab_8_15S, 24);
            cout << "Encoding at 8/15" << endl;
        }
        if (code_rate == gr::atsc32::C9_15)  {
            LDPC_BF(ldpc_tab_9_15S, 27);
            cout << "Encoding at 9/15" << endl;
        }

        if (code_rate == gr::atsc32::C10_15)  {
            LDPC_BF(ldpc_tab_10_15S, 30);
            cout << "Encoding at 10/15" << endl;
        }

        if (code_rate == gr::atsc32::C11_15)  {
            LDPC_BF(ldpc_tab_11_15S, 33);
            cout << "Encoding at 11/15" << endl;
        }
    }
    ldpc_encode.table_length = index;
}

    int
    ldpc_encoder_bb_impl::general_work (int noutput_items,
                       gr_vector_int &ninput_items,
                       gr_vector_const_void_star &input_items,
                       gr_vector_void_star &output_items)
    {
      const unsigned char *in = (const unsigned char *) input_items[0];
      unsigned char *out = (unsigned char *) output_items[0];
      const unsigned char *d;
      unsigned char *p;
      //numero de bits de paridad
      int plen = frame_size - nouter;
      d = in;
      p = &out[nouter];
      int consumed = 0;

        for (int i = 0; i < noutput_items; i += frame_size)
        {
            // First zero all the parity bits
            memset(p, 0, sizeof(unsigned char)*plen);
            for(int j = 0; j < (int)nouter; j++)
            {
                out[i + j] = in[consumed];
                consumed++;
            }
            // now do the parity checking
            for(int j = 0; j < ldpc_encode.table_length; j++)
            {
                p[ldpc_encode.p[j]] ^= d[ldpc_encode.d[j]];
            }
            for(int j = 1; j < plen; j++)
            {
                p[j] ^= p[j-1];
            }
            d += nouter;
            p += frame_size;
        }
      // Tell runtime system how many input items we consumed on
      // each input stream.
      consume_each (consumed);

      // Tell runtime system how many output items we produced.
      return noutput_items;
    }

const int ldpc_encoder_bb_impl::ldpc_tab_6_15N[72][28]=
{

        {27, 1606, 3402, 4961, 6751, 7132, 11516, 12300, 12482, 12592, 13342, 13764, 14123, 21576, 23946, 24533, 25376, 25667, 26836, 31799, 34173, 35462, 36153, 36740, 37085, 37152, 37468, 37658},
        {27, 4621, 5007, 6910, 8732, 9757, 11508, 13099, 15513, 16335, 18052, 19512, 21319, 23663, 25628, 27208, 31333, 32219, 33003, 33239, 33447, 36200, 36473, 36938, 37201, 37283, 37495, 38642},
        {27, 16, 1094, 2020, 3080, 4194, 5098, 5631, 6877, 7889, 8237, 9804, 10067, 11017, 11366, 13136, 13354, 15379, 18934, 20199, 24522, 26172, 28666, 30386, 32714, 36390, 37015, 37162},
        {27, 700, 897, 1708, 6017, 6490, 7372, 7825, 9546, 10398, 16605, 18561, 18745, 21625, 22137, 23693, 24340, 24966, 25015, 26995, 28586, 28895, 29687, 33938, 34520, 34858, 37056, 38297},
        {27, 159, 2010, 2573, 3617, 4452, 4958, 5556, 5832, 6481, 8227, 9924, 10836, 14954, 15594, 16623, 18065, 19249, 22394, 22677, 23408, 23731, 24076, 24776, 27007, 28222, 30343, 38371},
        {27, 3118, 3545, 4768, 4992, 5227, 6732, 8170, 9397, 10522, 11508, 15536, 20218, 21921, 28599, 29445, 29758, 29968, 31014, 32027, 33685, 34378, 35867, 36323, 36728, 36870, 38335, 38623},
        {27, 1264, 4254, 6936, 9165, 9486, 9950, 10861, 11653, 13697, 13961, 15164, 15665, 18444, 19470, 20313, 21189, 24371, 26431, 26999, 28086, 28251, 29261, 31981, 34015, 35850, 36129, 37186},
        {27, 111, 1307, 1628, 2041, 2524, 5358, 7988, 8191, 10322, 11905, 12919, 14127, 15515, 15711, 17061, 19024, 21195, 22902, 23727, 24401, 24608, 25111, 25228, 27338, 35398, 37794, 38196},
        {27, 961, 3035, 7174, 7948, 13355, 13607, 14971, 18189, 18339, 18665, 18875, 19142, 20615, 21136, 21309, 21758, 23366, 24745, 25849, 25982, 27583, 30006, 31118, 32106, 36469, 36583, 37920},
        {27, 2990, 3549, 4273, 4808, 5707, 6021, 6509, 7456, 8240, 10044, 12262, 12660, 13085, 14750, 15680, 16049, 21587, 23997, 25803, 28343, 28693, 34393, 34860, 35490, 36021, 37737, 38296},
        {27, 955, 4323, 5145, 6885, 8123, 9730, 11840, 12216, 19194, 20313, 23056, 24248, 24830, 25268, 26617, 26801, 28557, 29753, 30745, 31450, 31973, 32839, 33025, 33296, 35710, 37366, 37509},
        {27, 264, 605, 4181, 4483, 5156, 7238, 8863, 10939, 11251, 12964, 16254, 17511, 20017, 22395, 22818, 23261, 23422, 24064, 26329, 27723, 28186, 30434, 31956, 33971, 34372, 36764, 38123},
        {27, 520, 2562, 2794, 3528, 3860, 4402, 5676, 6963, 8655, 9018, 9783, 11933, 16336, 17193, 17320, 19035, 20606, 23579, 23769, 24123, 24966, 27866, 32457, 34011, 34499, 36620, 37526},
        {4, 10106, 10637, 10906, 34242, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 1856, 15100, 19378, 21848, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 943, 11191, 27806, 29411, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 4575, 6359, 13629, 19383, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 4476, 4953, 18782, 24313, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 5441, 6381, 21840, 35943, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 9638, 9763, 12546, 30120, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 9587, 10626, 11047, 25700, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 4088, 15298, 28768, 35047, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 2332, 6363, 8782, 28863, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 4625, 4933, 28298, 30289, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 3541, 4918, 18257, 31746, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 1221, 25233, 26757, 34892, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 8150, 16677, 27934, 30021, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 8500, 25016, 33043, 38070, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 7374, 10207, 16189, 35811, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 611, 18480, 20064, 38261, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 25416, 27352, 36089, 38469, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 1667, 17614, 25839, 32776, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 4118, 12481, 21912, 37945, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 5573, 13222, 23619, 31271, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 18271, 26251, 27182, 30587, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 14690, 26430, 26799, 34355, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 13688, 16040, 20716, 34558, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 2740, 14957, 23436, 32540, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 3491, 14365, 14681, 36858, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 4796, 6238, 25203, 27854, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 1731, 12816, 17344, 26025, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 19182, 21662, 23742, 27872, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 6502, 13641, 17509, 34713, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 12246, 12372, 16746, 27452, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 1589, 21528, 30621, 34003, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 12328, 20515, 30651, 31432, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 3415, 22656, 23427, 36395, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 632, 5209, 25958, 31085, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 619, 3690, 19648, 37778, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 9528, 13581, 26965, 36447, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 2147, 26249, 26968, 28776, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 15698, 18209, 30683, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 1132, 19888, 34111, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 4608, 25513, 38874, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 475, 1729, 34100, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 7348, 32277, 38587, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 182, 16473, 33082, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 3865, 9678, 21265, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 4447, 20151, 27618, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 6335, 14371, 38711, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 704, 9695, 28858, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 4856, 9757, 30546, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 1993, 19361, 30732, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 756, 28000, 29138, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 3821, 24076, 31813, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 4611, 12326, 32291, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 7628, 21515, 34995, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 1246, 13294, 30068, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 6466, 33233, 35865, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 14484, 23274, 38150, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 21269, 36411, 37450, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 23129, 26195, 37653, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}
};

const int ldpc_encoder_bb_impl::ldpc_tab_6_15S[18][31]=
{
        {30, 27, 430, 519, 828, 1897, 1943, 2513, 2600, 2640, 3310, 3415, 4266, 5044, 5100, 5328, 5483, 5928, 6204, 6392, 6416, 6602, 7019, 7415, 7623, 8112, 8485, 8724, 8994, 9445, 9667},
        {30, 27, 174, 188, 631, 1172, 1427, 1779, 2217, 2270, 2601, 2813, 3196, 3582, 3895, 3908, 3948, 4463, 4955, 5120, 5809, 5988, 6478, 6604, 7096, 7673, 7735, 7795, 8925, 9613, 9670},
        {30, 27, 370, 617, 852, 910, 1030, 1326, 1521, 1606, 2118, 2248, 2909, 3214, 3413, 3623, 3742, 3752, 4317, 4694, 5300, 5687, 6039, 6100, 6232, 6491, 6621, 6860, 7304, 8542, 8634},
        {4, 990, 1753, 7635, 8540, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 933, 1415, 5666, 8745, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 27, 6567, 8707, 9216, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 2341, 8692, 9580, 9615, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 260, 1092, 5839, 6080, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 352, 3750, 4847, 7726, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 4610, 6580, 9506, 9597, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 2512, 2974, 4814, 9348, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 1461, 4021, 5060, 7009, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 1796, 2883, 5553, 8306, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 1249, 5422, 7057, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 3965, 6968, 9422, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 1498, 2931, 5092, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 27, 1090, 6215, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 26, 4232, 6354, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}
};

const int ldpc_encoder_bb_impl::ldpc_tab_7_15S[21][25]=
{
        {24, 553, 742, 901, 1327, 1544, 2179, 2519, 3131, 3280, 3603, 3789, 3792, 4253, 5340, 5934, 5962, 6004, 6698, 7793, 8001, 8058, 8126, 8276, 8559},
        {24, 503, 590, 598, 1185, 1266, 1336, 1806, 2473, 3021, 3356, 3490, 3680, 3936, 4501, 4659, 5891, 6132, 6340, 6602, 7447, 8007, 8045, 8059, 8249},
        {24, 795, 831, 947, 1330, 1502, 2041, 2328, 2513, 2814, 2829, 4048, 4802, 6044, 6109, 6461, 6777, 6800, 7099, 7126, 8095, 8428, 8519, 8556, 8610},
        {24, 601, 787, 899, 1757, 2259, 2518, 2783, 2816, 2823, 2949, 3396, 4330, 4494, 4684, 4700, 4837, 4881, 4975, 5130, 5464, 6554, 6912, 7094, 8297},
        {4, 4229, 5628, 7917, 7992, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 1506, 3374, 4174, 5547, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 4275, 5650, 8208, 8533, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 1504, 1747, 3433, 6345, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 3659, 6955, 7575, 7852, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 607, 3002, 4913, 6453, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 3533, 6860, 7895, 8048, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 4094, 6366, 8314, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 2206, 4513, 5411, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 32, 3882, 5149, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 389, 3121, 4626, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 1308, 4419, 6520, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 2092, 2373, 6849, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 1815, 3679, 7152, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 3582, 3979, 6948, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 1049, 2135, 3754, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 2276, 4442, 6591, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}
};



const int ldpc_encoder_bb_impl::ldpc_tab_8_15N[96][20]=
{
        {19, 2768, 3039, 4059, 5856, 6245, 7013, 8157, 9341, 9802, 10470, 11521, 12083, 16610, 18361, 20321, 24601, 27420, 28206, 29788},
        {19, 2739, 8244, 8891, 9157, 12624, 12973, 15534, 16622, 16919, 18402, 18780, 19854, 20220, 20543, 22306, 25540, 27478, 27678, 28053},
        {19, 1727, 2268, 6246, 7815, 9010, 9556, 10134, 10472, 11389, 14599, 15719, 16204, 17342, 17666, 18850, 22058, 25579, 25860, 29207},
        {19, 28, 1346, 3721, 5565, 7019, 9240, 12355, 13109, 14800, 16040, 16839, 17369, 17631, 19357, 19473, 19891, 20381, 23911, 29683},
        {19, 869, 2450, 4386, 5316, 6160, 7107, 10362, 11132, 11271, 13149, 16397, 16532, 17113, 19894, 22043, 22784, 27383, 28615, 28804},
        {19, 508, 4292, 5831, 8559, 10044, 10412, 11283, 14810, 15888, 17243, 17538, 19903, 20528, 22090, 22652, 27235, 27384, 28208, 28485},
        {19, 389, 2248, 5840, 6043, 7000, 9054, 11075, 11760, 12217, 12565, 13587, 15403, 19422, 19528, 21493, 25142, 27777, 28566, 28702},
        {19, 1015, 2002, 5764, 6777, 9346, 9629, 11039, 11153, 12690, 13068, 13990, 16841, 17702, 20021, 24106, 26300, 29332, 30081, 30196},
        {19, 1480, 3084, 3467, 4401, 4798, 5187, 7851, 11368, 12323, 14325, 14546, 16360, 17158, 18010, 21333, 25612, 26556, 26906, 27005},
        {19, 6925, 8876, 12392, 14529, 15253, 15437, 19226, 19950, 20321, 23021, 23651, 24393, 24653, 26668, 27205, 28269, 28529, 29041, 29292},
        {19, 2547, 3404, 3538, 4666, 5126, 5468, 7695, 8799, 14732, 15072, 15881, 17410, 18971, 19609, 19717, 22150, 24941, 27908, 29018},
        {19, 888, 1581, 2311, 5511, 7218, 9107, 10454, 12252, 13662, 15714, 15894, 17025, 18671, 24304, 25316, 25556, 28489, 28977, 29212},
        {19, 1047, 1494, 1718, 4645, 5030, 6811, 7868, 8146, 10611, 15767, 17682, 18391, 22614, 23021, 23763, 25478, 26491, 29088, 29757},
        {19, 59, 1781, 1900, 3814, 4121, 8044, 8906, 9175, 11156, 14841, 15789, 16033, 16755, 17292, 18550, 19310, 22505, 29567, 29850},
        {19, 1952, 3057, 4399, 9476, 10171, 10769, 11335, 11569, 15002, 19501, 20621, 22642, 23452, 24360, 25109, 25290, 25828, 28505, 29122},
        {19, 2895, 3070, 3437, 4764, 4905, 6670, 9244, 11845, 13352, 13573, 13975, 14600, 15871, 17996, 19672, 20079, 20579, 25327, 27958},
        {19, 612, 1528, 2004, 4244, 4599, 4926, 5843, 7684, 10122, 10443, 12267, 14368, 18413, 19058, 22985, 24257, 26202, 26596, 27899},
        {19, 1361, 2195, 4146, 6708, 7158, 7538, 9138, 9998, 14862, 15359, 16076, 18925, 21401, 21573, 22503, 24146, 24247, 27778, 29312},
        {19, 5229, 6235, 7134, 7655, 9139, 13527, 15408, 16058, 16705, 18320, 19909, 20901, 22238, 22437, 23654, 25131, 27550, 28247, 29903},
        {19, 697, 2035, 4887, 5275, 6909, 9166, 11805, 15338, 16381, 18403, 20425, 20688, 21547, 24590, 25171, 26726, 28848, 29224, 29412},
        {4, 5379, 17329, 22659, 23062, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 11814, 14759, 22329, 22936, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 2423, 2811, 10296, 12727, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 8460, 15260, 16769, 17290, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 14191, 14608, 29536, 30187, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 7103, 10069, 20111, 22850, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 4285, 15413, 26448, 29069, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 548, 2137, 9189, 109280, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 4581, 7077, 23382, 23949, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 3942, 17248, 19486, 27922, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 8668, 10230, 16922, 26678, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 6158, 9980, 13788, 28198, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 12422, 16076, 24206, 29887, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 8778, 10649, 18747, 22111, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 21029, 22677, 27150, 28980, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 7918, 15423, 27672, 27803, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 5927, 18086, 23525, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 3397, 15058, 30224, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 24016, 25880, 26268, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 1096, 4775, 7912, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 3259, 17301, 20802, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 129, 8396, 15132, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 17825, 28119, 28676, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 2343, 8382, 28840, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 3907, 18374, 20939, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 1132, 1290, 8786, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 1481, 4710, 28846, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 2185, 3705, 26834, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 5496, 15681, 21854, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 12697, 13407, 22178, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 12788, 21227, 22894, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 629, 2854, 6232, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 2289, 18227, 27458, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 7593, 21935, 23001, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 3836, 7081, 12282, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 7925, 18440, 23135, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 497, 6342, 9717, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 11199, 22046, 30067, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 12572, 28045, 28990, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 1240, 2023, 10933, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 19566, 20629, 25186, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 6442, 13303, 28813, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 4765, 10572, 16180, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 552, 19301, 24286, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 6782, 18480, 21383, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 11267, 12288, 15758, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 771, 5652, 15531, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 16131, 20047, 25649, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 13227, 23035, 24450, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 4839, 13467, 27488, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 2852, 4677, 22993, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 2504, 28116, 29524, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 12518, 17374, 24267, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 1222, 11859, 27922, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 9660, 17286, 18261, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 232, 11296, 29978, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 9750, 11165, 16295, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 4894, 9505, 23622, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 10861, 11980, 14110, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 2128, 15883, 22836, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 6274, 17243, 21989, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 10866, 13202, 22517, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 11159, 16111, 21608, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 3719, 18787, 22100, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 1756, 2020, 23901, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 20913, 29473, 30103, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 2729, 15091, 26976, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 4410, 8217, 12963, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 5395, 24564, 28235, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 3859, 17909, 23051, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 5733, 26005, 29797, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 1935, 3492, 29773, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 11903, 21380, 29914, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 6091, 10469, 29997, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 2895, 8930, 15594, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 1827, 10028, 20070, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}
};

const int ldpc_encoder_bb_impl::ldpc_tab_8_15S[24][33]=
{

        {32, 5, 519, 825, 1871, 2098, 2478, 2659, 2820, 3200, 3294, 3650, 3804, 3949, 4426, 4460, 4503, 4568, 4590, 4949, 5219, 5662, 5738, 5905, 5911, 6160, 6404, 6637, 6708, 6737, 6814, 7263, 7412},
        {32, 81, 391, 1272, 1633, 2062, 2882, 3443, 3503, 3535, 3908, 4033, 4163, 4490, 4929, 5262, 5399, 5576, 5768, 5910, 6331, 6430, 6844, 6867, 7201, 7274, 7290, 7343, 7350, 7378, 7387, 7440, 7554},
        {14, 105, 975, 3421, 3480, 4120, 4444, 5957, 5971, 6119, 6617, 6761, 6810, 7067, 7353, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {14, 6, 138, 485, 1444, 1512, 2615, 2990, 3109, 5604, 6435, 6513, 6632, 6704, 7507, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {14, 20, 858, 1051, 2539, 3049, 5162, 5308, 6158, 6391, 6604, 6744, 7071, 7195, 7238, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 1140, 5838, 6203, 6748, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 6282, 6466, 6481, 6638, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 2346, 2592, 5436, 7487, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 2219, 3897, 5896, 7528, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 2897, 6028, 7018, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 1285, 1863, 5324, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 3075, 6005, 6466, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 5, 6020, 7551, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 2121, 3751, 7507, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 4027, 5488, 7542, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 2, 6012, 7011, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 3823, 5531, 5687, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 1379, 2262, 5297, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 1882, 7498, 7551, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 3749, 4806, 7227, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 2, 2074, 6898, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 17, 616, 7482, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 9, 6823, 7480, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 5195, 5880, 7559, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}
};

const int ldpc_encoder_bb_impl::ldpc_tab_9_15N[108][20]=
{
        {19, 113, 1557, 3316, 5680, 6241, 10407, 13404, 13947, 14040, 14353, 15522, 15698, 16079, 17363, 19374, 19543, 20530, 22833, 24339},
        {19, 271, 1361, 6236, 7006, 7307, 7333, 12768, 15441, 15568, 17923, 18341, 20321, 21502, 22023, 23938, 25351, 25590, 25876, 25910},
        {19, 73, 605, 872, 4008, 6279, 7653, 10346, 10799, 12482, 12935, 13604, 15909, 16526, 19782, 20506, 22804, 23629, 24859, 25600},
        {19, 1445, 1690, 4304, 4851, 8919, 9176, 9252, 13783, 16076, 16675, 17274, 18806, 18882, 20819, 21958, 22451, 23869, 23999, 24177},
        {19, 1290, 2337, 5661, 6371, 8996, 10102, 10941, 11360, 12242, 14918, 16808, 20571, 23374, 24046, 25045, 25060, 25662, 25783, 25913},
        {19, 28, 42, 1926, 3421, 3503, 8558, 9453, 10168, 15820, 17473, 19571, 19685, 22790, 23336, 23367, 23890, 24061, 25657, 25680},
        {19, 0, 1709, 4041, 4932, 5968, 7123, 8430, 9564, 10596, 11026, 14761, 19484, 20762, 20858, 23803, 24016, 24795, 25853, 25863},
        {19, 29, 1625, 6500, 6609, 16831, 18517, 18568, 18738, 19387, 20159, 20544, 21603, 21941, 24137, 24269, 24416, 24803, 25154, 25395},
        {19, 55, 66, 871, 3700, 11426, 13221, 15001, 16367, 17601, 18380, 22796, 23488, 23938, 25476, 25635, 25678, 25807, 25857, 25872},
        {19, 1, 19, 5958, 8548, 8860, 11489, 16845, 18450, 18469, 19496, 20190, 23173, 25262, 25566, 25668, 25679, 25858, 25888, 25915},
        {19, 7520, 7690, 8855, 9183, 14654, 16695, 17121, 17854, 18083, 18428, 19633, 20470, 20736, 21720, 22335, 23273, 25083, 25293, 25403},
        {19, 48, 58, 410, 1299, 3786, 10668, 18523, 18963, 20864, 22106, 22308, 23033, 23107, 23128, 23990, 24286, 24409, 24595, 25802},
        {19, 12, 51, 3894, 6539, 8276, 10885, 11644, 12777, 13427, 14039, 15954, 17078, 19053, 20537, 22863, 24521, 25087, 25463, 25838},
        {19, 3509, 8748, 9581, 11509, 15884, 16230, 17583, 19264, 20900, 21001, 21310, 22547, 22756, 22959, 24768, 24814, 25594, 25626, 25880},
        {19, 21, 29, 69, 1448, 2386, 4601, 6626, 6667, 10242, 13141, 13852, 14137, 18640, 19951, 22449, 23454, 24431, 25512, 25814},
        {19, 18, 53, 7890, 9934, 10063, 16728, 19040, 19809, 20825, 21522, 21800, 23582, 24556, 25031, 25547, 25562, 25733, 25789, 25906},
        {19, 4096, 4582, 5766, 5894, 6517, 10027, 12182, 13247, 15207, 17041, 18958, 20133, 20503, 22228, 24332, 24613, 25689, 25855, 25883},
        {19, 0, 25, 819, 5539, 7076, 7536, 7695, 9532, 13668, 15051, 17683, 19665, 20253, 21996, 24136, 24890, 25758, 25784, 25807},
        {19, 34, 40, 44, 4215, 6076, 7427, 7965, 8777, 11017, 15593, 19542, 22202, 22973, 23397, 23423, 24418, 24873, 25107, 25644},
        {4, 1595, 6216, 22850, 25439, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 1562, 15172, 19517, 22362, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 7508, 12879, 24324, 24496, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 6298, 15819, 16757, 18721, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 11173, 15175, 19966, 21195, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 59, 13505, 16941, 23793, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 2267, 4830, 12023, 20587, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 8827, 9278, 13072, 16664, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 14419, 17463, 23398, 25348, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 6112, 16534, 20423, 22698, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 493, 8914, 21103, 24799, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 6896, 12761, 13206, 25873, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 2, 1380, 12322, 21701, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 11600, 21306, 25753, 25790, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 8421, 13076, 14271, 15401, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 9630, 14112, 19017, 20955, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 212, 13932, 21781, 25824, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 5961, 9110, 16654, 19636, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 58, 5434, 9936, 12770, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 6575, 11433, 19798, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 2731, 7338, 20926, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 14253, 18463, 25404, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 21791, 24805, 25869, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 2, 11646, 15850, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 6075, 8586, 23819, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 18435, 22093, 24852, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 2103, 2368, 11704, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 10925, 17402, 18232, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 9062, 25061, 25674, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 18497, 20853, 23404, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 18606, 19364, 19551, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 7, 1022, 25543, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 6744, 15481, 25868, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 9081, 17305, 25164, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 8, 23701, 25883, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 9680, 19955, 22848, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 56, 4564, 19121, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 5595, 15086, 25892, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 3174, 17127, 23183, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 19397, 19817, 20275, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 12561, 24571, 25825, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 7111, 9889, 25865, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 19104, 20189, 21851, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 549, 9686, 25548, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 6586, 20325, 25906, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 3224, 20710, 21637, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 641, 15215, 25754, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 13484, 23729, 25818, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 2043, 7493, 24246, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 16860, 25230, 25768, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 22047, 24200, 24902, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 9391, 18040, 19499, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 7855, 24336, 25069, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 23834, 25570, 25852, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 1977, 8800, 25756, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 6671, 21772, 25859, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 3279, 6710, 24444, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 24099, 25117, 25820, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 5553, 12306, 25915, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 48, 11107, 23907, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 10832, 11974, 25773, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 2223, 17905, 25484, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 16782, 17135, 20446, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 475, 2861, 3457, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 16218, 22449, 24362, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 11716, 22200, 25897, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 8315, 15009, 22633, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 13, 20480, 25852, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 12352, 18658, 25687, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 3681, 14794, 23703, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 30, 24531, 25846, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 4103, 22077, 24107, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 23837, 25622, 25812, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 3627, 13387, 25839, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 908, 5367, 19388, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 0, 6894, 25795, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 20322, 23546, 25181, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 8178, 25260, 25437, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 2449, 13244, 22565, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 31, 18928, 22741, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 1312, 5134, 14838, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 6085, 13937, 24220, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 66, 14633, 25670, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 47, 22512, 25472, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 8867, 24704, 25279, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 6742, 21623, 22745, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 147, 9948, 24178, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 8522, 24261, 24307, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 19202, 22406, 24609, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}
};

const int ldpc_encoder_bb_impl::ldpc_tab_9_15S[27][17]=
{
        {16, 212, 255, 540, 967, 1033, 1517, 1538, 3124, 3408, 3800, 4373, 4864, 4905, 5163, 5177, 6186},
        {16, 275, 660, 1351, 2211, 2876, 3063, 3433, 4088, 4273, 4544, 4618, 4632, 5548, 6101, 6111, 6136},
        {16, 279, 335, 494, 865, 1662, 1681, 3414, 3775, 4252, 4595, 5272, 5471, 5796, 5907, 5986, 6008},
        {10, 345, 352, 3094, 3188, 4297, 4338, 4490, 4865, 5303, 6477, 0, 0, 0, 0, 0, 0},
        {10, 222, 681, 1218, 3169, 3850, 4878, 4954, 5666, 6001, 6237, 0, 0, 0, 0, 0, 0},
        {9, 172, 512, 1536, 1559, 2179, 2227, 3334, 4049, 6464, 0, 0, 0, 0, 0, 0, 0},
        {9, 716, 934, 1694, 2890, 3276, 3608, 4332, 4468, 5945, 0, 0, 0, 0, 0, 0, 0},
        {9, 1133, 1593, 1825, 2571, 3017, 4251, 5221, 5639, 5845, 0, 0, 0, 0, 0, 0, 0},
        {3, 1076, 1222, 6465, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 159, 5064, 6078, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 374, 4073, 5357, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 2833, 5526, 5845, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 1594, 3639, 5419, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 1028, 1392, 4239, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 115, 622, 2175, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 300, 1748, 6245, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 2724, 3276, 5349, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 1433, 6117, 6448, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 485, 663, 4955, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 711, 1132, 4315, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 177, 3266, 4339, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 1171, 4841, 4982, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 33, 1584, 3692, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 2820, 3485, 4249, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 1716, 2428, 3125, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 250, 2275, 6338, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 108, 1719, 4961, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}

};

const int ldpc_encoder_bb_impl::ldpc_tab_10_15N[120][15]=
{

        {14, 316, 1271, 3692, 9495, 12147, 12849, 14928, 16671, 16938, 17864, 19108, 20502, 21097, 21115},
        {14, 2341, 2559, 2643, 2816, 2865, 5137, 5331, 7000, 7523, 8023, 10439, 10797, 13208, 15041},
        {14, 5556, 6858, 7677, 10162, 10207, 11349, 12321, 12398, 14787, 15743, 15859, 15952, 19313, 20879},
        {14, 349, 573, 910, 2702, 3654, 6214, 9246, 9353, 10638, 11772, 14447, 14953, 16620, 19888},
        {14, 204, 1390, 2887, 3835, 6230, 6533, 7443, 7876, 9299, 10291, 10896, 13960, 18287, 20086},
        {14, 541, 2429, 2838, 7144, 8523, 8637, 10490, 10585, 11074, 12074, 15762, 16812, 17900, 18548},
        {14, 733, 1659, 3838, 5323, 5805, 7882, 9429, 10682, 13697, 16909, 18846, 19587, 19592, 20904},
        {14, 1134, 2136, 4631, 4653, 4718, 5197, 10410, 11666, 14996, 15305, 16048, 17417, 18960, 20303},
        {14, 734, 1001, 1283, 4959, 10016, 10176, 10973, 11578, 12051, 15550, 15915, 19022, 19430, 20121},
        {14, 745, 4057, 5855, 9885, 10594, 10989, 13156, 13219, 13351, 13631, 13685, 14577, 17713, 20386},
        {14, 968, 1446, 2130, 2502, 3092, 3787, 5323, 8104, 8418, 9998, 11681, 13972, 17747, 17929},
        {14, 3020, 3857, 5275, 5786, 6319, 8608, 11943, 14062, 17144, 17752, 18001, 18453, 19311, 21414},
        {14, 709, 747, 1038, 2181, 5320, 8292, 10584, 10859, 13964, 15009, 15277, 16953, 20675, 21509},
        {14, 1663, 3247, 5003, 5760, 7186, 7360, 10346, 14211, 14717, 14792, 15155, 16128, 17355, 17970},
        {14, 516, 578, 1914, 6147, 9419, 11148, 11434, 13289, 13325, 13332, 19106, 19257, 20962, 21556},
        {14, 5009, 5632, 6531, 9430, 9886, 10621, 11765, 13969, 16178, 16413, 18110, 18249, 20616, 20759},
        {14, 457, 2686, 3318, 4608, 5620, 5858, 6480, 7430, 9602, 12691, 14664, 18777, 20152, 20848},
        {14, 33, 2877, 5334, 6851, 7907, 8654, 10688, 15401, 16123, 17942, 17969, 18747, 18931, 20224},
        {14, 87, 897, 7636, 8663, 11425, 12288, 12672, 14199, 16435, 17615, 17950, 18953, 19667, 20281},
        {14, 1042, 1832, 2545, 2719, 2947, 3672, 3700, 6249, 6398, 6833, 11114, 14283, 17694, 20477},
        {14, 326, 488, 2662, 2880, 3009, 5357, 6587, 8882, 11604, 14374, 18781, 19051, 19057, 20508},
        {14, 854, 1294, 2436, 2852, 4903, 6466, 7761, 9072, 9564, 10321, 13638, 15658, 16946, 19119},
        {14, 194, 899, 1711, 2408, 2786, 5391, 7108, 8079, 8716, 11453, 17303, 19484, 20989, 21389},
        {14, 1631, 3121, 3994, 5005, 7810, 8850, 10315, 10589, 13407, 17162, 18624, 18758, 19311, 20301},
        {9, 736, 2424, 4792, 5600, 6370, 10061, 16053, 16775, 18600, 0, 0 ,0 ,0 ,0},
        {9, 1254, 8163, 8876, 9157, 12141, 14587, 16545, 17175, 18191, 0, 0 ,0 ,0 ,0},
        {9, 388, 6641, 8974, 10607, 10716, 14477, 16825, 17191, 18400, 0, 0 ,0 ,0 ,0},
        {9, 5578, 6082, 6824, 7360, 7745, 8655, 11402, 11665, 12428, 0, 0 ,0 ,0 ,0},
        {9, 3603, 8729, 13463, 14698, 15210, 19112, 19550, 20727, 21052, 0, 0 ,0 ,0 ,0},
        {8, 1732, 3805, 5158, 15442, 16909, 19854, 21071, 21579, 0, 0 ,0 ,0 ,0, 0},
        {3, 11707, 14014, 21531, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 1542, 4133, 4925, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 10083, 13505, 21198, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 14300, 15765, 16752, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 778, 1237, 11215, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 1325, 3199, 14534, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 2007, 14510, 20599, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 1996, 5881, 16429, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 5111, 15018, 15980, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 4989, 10681, 12810, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 3763, 10715, 16515, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 2259, 10080, 15642, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 9032, 11319, 21305, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 3915, 15213, 20884, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 11150, 15022, 20201, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 1147, 6749, 19625, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 12139, 12939, 18870, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 3840, 4634, 10244, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 1018, 10231, 17720, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 2708, 13056, 13393, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 5781, 11588, 18888, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 1345, 2036, 5252, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 5908, 8143, 15141, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 1804, 13693, 18640, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 10433, 13965, 16950, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 9568, 10122, 15945, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 547, 6722, 14015, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 321, 12844, 14095, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 2632, 10513, 14936, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 6369, 11995, 20321, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 9920, 19136, 21529, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 1990, 2726, 10183, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 5763, 12118, 15467, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 503, 10006, 19564, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 9839, 11942, 19472, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 11205, 13552, 15389, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 8841, 13797, 19697, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 124, 6053, 18224, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 6477, 14406, 21146, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 1224, 8027, 16011, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 3046, 4422, 17717, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 739, 12308, 17760, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 4014, 4130, 7835, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 2266, 5652, 11981, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 2711, 7970, 18317, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 2196, 15229, 17217, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 8636, 13302, 16764, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 5612, 15010, 16657, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 615, 1249, 4639, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 3821, 12073, 18506, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 1066, 16522, 21536, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 11307, 18363, 19740, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 3240, 8560, 10391, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 3124, 11424, 20779, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 1604, 8861, 17394, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 2083, 7400, 8093, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 3218, 7454, 9155, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 9855, 15998, 20533, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 316, 2850, 20652, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 5583, 9768, 10333, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 7147, 7713, 18339, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 12607, 17428, 21418, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 14216, 16954, 18164, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 8477, 15970, 18488, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 1632, 8032, 9751, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 4573, 9080, 13507, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 11747, 12441, 13876, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 1183, 15605, 16675, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 4408, 10264, 17109, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 5495, 7882, 12150, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 1010, 3763, 5065, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 9828, 18054, 21599, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 6342, 7353, 15358, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 6362, 9462, 19999, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 7184, 13693, 17622, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 4343, 4654, 10995, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 7099, 8466, 18520, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 11505, 14395, 15138, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 6779, 16691, 18726, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 7146, 12644, 20196, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 5865, 16728, 19634, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 4657, 8714, 21246, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 4580, 5279, 18750, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 3767, 6620, 18905, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 9209, 13093, 17575, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 12486, 15875, 19791, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 8046, 14636, 17491, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 2120, 4643, 13206, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 6186, 9675, 12601, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {3, 784, 5770, 21585, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0}
};

const int ldpc_encoder_bb_impl::ldpc_tab_10_15S[30][26]=
{
        {25, 352, 747, 894, 1437, 1688, 1807, 1883, 2119, 2159, 3321, 3400, 3543, 3588, 3770, 3821, 4384, 4470, 4884, 5012, 5036, 5084, 5101, 5271, 5281, 5353},
        {25, 505, 915, 1156, 1269, 1518, 1650, 2153, 2256, 2344, 2465, 2509, 2867, 2875, 3007, 3254, 3519, 3687, 4331, 4439, 4532, 4940, 5011, 5076, 5113, 5367},
        {14, 268, 346, 650, 919, 1260, 4389, 4653, 4721, 4838, 5054, 5157, 5162, 5275, 5362, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {14, 220, 236, 828, 1590, 1792, 3259, 3647, 4276, 4281, 4325, 4963, 4974, 5003, 5037, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {14, 381, 737, 1099, 1409, 2364, 2955, 3228, 3341, 3473, 3985, 4257, 4730, 5173, 5242, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {14, 88, 771, 1640, 1737, 1803, 2408, 2575, 2974, 3167, 3464, 3780, 4501, 4901, 5047, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0},
        {4, 749, 1502, 2201, 3189, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 2873, 3245, 3427, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 2158, 2605, 3165, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 1, 3438, 3606, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 10, 3019, 5221, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 371, 2901, 2923, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 9, 3935, 4683, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 1937, 3502, 3735, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 507, 3128, 4994, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 25, 3854, 4550, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 1178, 4737, 5366, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 2, 223, 5304, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 1146, 5175, 5197, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 1816, 2313, 3649, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 740, 1951, 3844, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 1320, 3703, 4791, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 1754, 2905, 4058, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 7, 917, 5277, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 3048, 3954, 5396, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 4804, 4824, 5105, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 2812, 3895, 5226, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 0, 5318, 5358, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 1483, 2324, 4826, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 2266, 4752, 5387, 0, 0 ,0 ,0 ,0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}
};

const int ldpc_encoder_bb_impl::ldpc_tab_11_15N[132][16]=
{

        {15, 696, 989, 1238, 3091, 3116, 3738, 4269, 6406, 7033, 8048, 9157, 10254, 12033, 16456, 16912},
        {15, 444, 1488, 6541, 8626, 10735, 12447, 13111, 13706, 14135, 15195, 15947, 16453, 16916, 17137, 17268},
        {15, 401, 460, 992, 1145, 1576, 1678, 2238, 2320, 4280, 6770, 10027, 12486, 15363, 16714, 17157},
        {15, 1161, 3108, 3727, 4508, 5092, 5348, 5582, 7727, 11793, 12515, 12917, 13362, 14247, 16717, 17205},
        {15, 542, 1190, 6883, 7911, 8349, 8835, 10489, 11631, 14195, 15009, 15454, 15482, 16632, 17040, 17063},
        {15, 17, 487, 776, 880, 5077, 6172, 9771, 11446, 12798, 16016, 16109, 16171, 17087, 17132, 17226},
        {15, 1337, 3275, 3462, 4229, 9246, 10180, 10845, 10866, 12250, 13633, 14482, 16024, 16812, 17186, 17241},
        {15, 15, 980, 2305, 3674, 5971, 8224, 11499, 11752, 11770, 12897, 14082, 14836, 15311, 16391, 17209},
        {15, 0, 3926, 5869, 8696, 9351, 9391, 11371, 14052, 14172, 14636, 14974, 16619, 16961, 17033, 17237},
        {15, 3033, 5317, 6501, 8579, 10698, 12168, 12966, 14019, 15392, 15806, 15991, 16493, 16690, 17062, 17090},
        {15, 981, 1205, 4400, 6410, 11003, 13319, 13405, 14695, 15846, 16297, 16492, 16563, 16616, 16862, 16953},
        {15, 1725, 4276, 8869, 9588, 14062, 14486, 15474, 15548, 16300, 16432, 17042, 17050, 17060, 17175, 17273},
        {15, 1807, 5921, 9960, 10011, 14305, 14490, 14872, 15852, 16054, 16061, 16306, 16799, 16833, 17136, 17262},
        {15, 2826, 4752, 6017, 6540, 7016, 8201, 14245, 14419, 14716, 15983, 16569, 16652, 17171, 17179, 17247},
        {15, 1662, 2516, 3345, 5229, 8086, 9686, 11456, 12210, 14595, 15808, 16011, 16421, 16825, 17112, 17195},
        {15, 2890, 4821, 5987, 7226, 8823, 9869, 12468, 14694, 15352, 15805, 16075, 16462, 17102, 17251, 17263},
        {15, 3751, 3890, 4382, 5720, 10281, 10411, 11350, 12721, 13121, 14127, 14980, 15202, 15335, 16735, 17123},
        {15, 26, 30, 2805, 5457, 6630, 7188, 7477, 7556, 11065, 16608, 16859, 16909, 16943, 17030, 17103},
        {15, 40, 4524, 5043, 5566, 9645, 10204, 10282, 11696, 13080, 14837, 15607, 16274, 17034, 17225, 17266},
        {15, 904, 3157, 6284, 7151, 7984, 11712, 12887, 13767, 15547, 16099, 16753, 16829, 17044, 17250, 17259},
        {15, 7, 311, 4876, 8334, 9249, 11267, 14072, 14559, 15003, 15235, 15686, 16331, 17177, 17238, 17253},
        {15, 4410, 8066, 8596, 9631, 10369, 11249, 12610, 15769, 16791, 16960, 17018, 17037, 17062, 17165, 17204},
        {15, 24, 8261, 9691, 10138, 11607, 12782, 12786, 13424, 13933, 15262, 15795, 16476, 17084, 17193, 17220},
        {4, 88, 11622, 14705, 15890, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 304, 2026, 2638, 6018, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 1163, 4268, 11620, 17232, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 9701, 11785, 14463, 17260, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 4118, 10952, 12224, 17006, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 3647, 10823, 11521, 12060, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {4, 1717, 3753, 9199, 11642, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 2187, 14280, 17220, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 14787, 16903, 17061, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 381, 3534, 4294, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 3149, 6947, 8323, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 12562, 16724, 16881, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 7289, 9997, 15306, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 5615, 13152, 17260, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 5666, 16926, 17027, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 4190, 7798, 16831, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 4778, 10629, 17180, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 10001, 13884, 15453, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 6, 2237, 8203, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 7831, 15144, 15160, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 9186, 17204, 17243, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 9435, 17168, 17237, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 42, 5701, 17159, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 7812, 14259, 15715, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 39, 4513, 6658, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 38, 9368, 11273, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 1119, 4785, 17182, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 5620, 16521, 16729, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 16, 6685, 17242, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 210, 3452, 12383, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 466, 14462, 16250, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 10548, 12633, 13962, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 1452, 6005, 16453, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 22, 4120, 13684, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 5195, 11563, 16522, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 5518, 16705, 17201, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 12233, 14552, 15471, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 6067, 13440, 17248, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 8660, 8967, 17061, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 8673, 12176, 15051, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 5959, 15767, 16541, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 3244, 12109, 12414, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 31, 15913, 16323, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 3270, 15686, 16653, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 24, 7346, 14675, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 12, 1531, 8740, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 6228, 7565, 16667, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 16936, 17122, 17162, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 4868, 8451, 13183, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 3714, 4451, 16919, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 11313, 13801, 17132, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 17070, 17191, 17242, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 1911, 11201, 17186, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 14, 17190, 17254, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 11760, 16008, 16832, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 14543, 17033, 17278, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 16129, 16765, 17155, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 6891, 15561, 17007, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 12741, 14744, 17116, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 8992, 16661, 17277, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 1861, 11130, 16742, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 4822, 13331, 16192, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 13281, 14027, 14989, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 38, 14887, 17141, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 10698, 13452, 15674, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 4, 2539, 16877, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 857, 17170, 17249, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 11449, 11906, 12867, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 285, 14118, 16831, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 15191, 17214, 17242, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 39, 728, 16915, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 2469, 12969, 15579, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 16644, 17151, 17164, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 2592, 8280, 10448, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 9236, 12431, 17173, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 9064, 16892, 17233, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 4526, 16146, 17038, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 31, 2116, 16083, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 15837, 16951, 17031, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 5362, 8382, 16618, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 6137, 13199, 17221, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 2841, 15068, 17068, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 24, 3620, 17003, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 9880, 15718, 16764, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 1784, 10240, 17209, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 2731, 10293, 10846, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 3121, 8723, 16598, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 8563, 15662, 17088, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 13, 1167, 14676, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 29, 13850, 15963, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 3654, 7553, 8114, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 23, 4362, 14865, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 4434, 14741, 16688, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 8362, 13901, 17244, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 13687, 16736, 17232, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 46, 4229, 13394, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 13169, 16383, 16972, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 16031, 16681, 16952, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 3384, 9894, 12580, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 9841, 14414, 16165, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 5013, 17099, 17115, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 2130, 8941, 17266, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 6907, 15428, 17241, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 16, 1860, 17235, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 2151, 16014, 16643, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 14954, 15958, 17222, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 3969, 8419, 15116, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 31, 15593, 16984, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 11514, 16605, 17255, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0}

};

const int ldpc_encoder_bb_impl::ldpc_tab_11_15S[33][13]=
{
        {12, 49, 719, 784, 794, 968, 2382, 2685, 2873, 2974, 2995, 3540, 4179},
        {12, 272, 281, 374, 1279, 2034, 2067, 2112, 3429, 3613, 3815, 3838, 4216},
        {12, 206, 714, 820, 1800, 1925, 2147, 2168, 2769, 2806, 3253, 3415, 4311},
        {12, 62, 159, 166, 605, 1496, 1711, 2652, 3016, 3347, 3517, 3654, 4113},
        {12, 363, 733, 1118, 2062, 2613, 2736, 3143, 3427, 3664, 4100, 4157, 4314},
        {12, 57, 142, 436, 983, 1364, 2105, 2113, 3074, 3639, 3835, 4164, 4242},
        {12, 870, 921, 950, 1212, 1861, 2128, 2707, 2993, 3730, 3968, 3983, 4227},
        {3, 185, 2684, 3263, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 2035, 2123, 2913, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 883, 2221, 3521, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 1344, 1773, 4132, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 438, 3178, 3650, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 543, 756, 1639, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 1057, 2337, 2898, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 171, 3298, 3929, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 1626, 2960, 3503, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 484, 3050, 3323, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 2283, 2336, 4189, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 2732, 4132, 4318, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 225, 2335, 3497, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 600, 2246, 2658, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 1240, 2790, 3020, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 301, 1097, 3539, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 1222, 1267, 2594, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 1364, 2004, 3603, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 1142, 1185, 2147, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 564, 1505, 2086, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 697, 991, 2908, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 1467, 2073, 3462, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 2574, 2818, 3637, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 748, 2577, 2772, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 1151, 1419, 4129, 0, 0, 0, 0, 0, 0, 0, 0, 0},
        {3, 164, 1238, 3401, 0, 0, 0, 0, 0, 0, 0, 0, 0}
};

} /* namespace atsc32 */
} /* namespace gr */

